/***************************************************
//
// An experimental wrapper class that implements the improved lux and color temperature from
// TAOS and a basic autorange mechanism.
//
// Written by ductsoup, public domain
//

// RGB Color Sensor with IR filter and White LED - TCS34725
// I2C 7-bit address 0x29, 8-bit address 0x52
//
// http://www.adafruit.com/product/1334
// http://learn.adafruit.com/adafruit-color-sensors/overview
// http://www.adafruit.com/datasheets/TCS34725.pdf
// http://www.ams.com/eng/Products/Light-Sensors/Color-Sensor/TCS34725
// http://www.ams.com/eng/content/view/download/265215 <- DN40, calculations
// http://www.ams.com/eng/content/view/download/181895 <- DN39, some thoughts on autogain
// http://www.ams.com/eng/content/view/download/145158 <- DN25 (original Adafruit calculations)
//

 MTSautoRange
 /* Connect SCL    to SCL
   Connect SDA    to SDA
   Connect VDD    to 3.3V DC
   Connect GROUND to common ground */

#include <Wire.h>
#include <SPI.h>
#include <SD.h>

#include "Adafruit_GFX.h"
#include "Adafruit_HX8357.h"
#include "Adafruit_TCS34725.h"

#if defined __SAMD51__
   #define STMPE_CS 6
   #define TFT_CS   9
   #define TFT_DC   10
   #define SD_CS    5
#endif

#define TFT_RST -1

// Use hardware SPI and the above for CS/DC
Adafruit_HX8357 tft = Adafruit_HX8357(TFT_CS, TFT_DC, TFT_RST);

// /* Initialise with specific int time and gain values */
// Adafruit_TCS34725 tcs = Adafruit_TCS34725(TCS34725_INTEGRATIONTIME_700MS, TCS34725_GAIN_1X);

#define LEDpin 12   // Turn off TCS34725 LED
#define storeIT 13  // Store button for file record
int counter;  //file counter
const int chipSelect = 5;  // Chip select for SD
File MISCFile;

//int RVal, GVal, BVal;
int MaxV = 65535; //Maximum RGB value
uint16_t red, grn, blu, clr, colorTemp, lux;

// Prototypes
void drawGraph();
void graph();
void writeFile();

// some magic numbers for this device from the DN40 application note
#define TCS34725_R_Coef 0.136
#define TCS34725_G_Coef 1.000
#define TCS34725_B_Coef -0.444
#define TCS34725_GA 1.0
#define TCS34725_DF 310.0
#define TCS34725_CT_Coef 3810.0
#define TCS34725_CT_Offset 1391.0

// Autorange class for TCS34725
class tcs34725 {
private:
  struct tcs_agc {
    tcs34725Gain_t ag;
    tcs34725IntegrationTime_t at;
    uint16_t mincnt;
    uint16_t maxcnt;
  };
  static const tcs_agc agc_lst[];
  uint16_t agc_cur;

  void setGainTime(void);
  Adafruit_TCS34725 tcs;

public:
  tcs34725(void);
											

  boolean begin(void);
  void getData(void);
			 

  boolean isAvailable, isSaturated;
  uint16_t againx, atime, atime_ms;
  uint16_t r, g, b, c;
  uint16_t ir;
  uint16_t r_comp, g_comp, b_comp, c_comp;
  uint16_t saturation, saturation75;
  float cratio, cpl, ct, lux, maxlux;
};
//
// Gain/time combinations to use and the min/max limits for hysteresis
// that avoid saturation. They should be in order from dim to bright.
//
// Also set the first min count and the last max count to 0 to indicate
// the start and end of the list.
//
const tcs34725::tcs_agc tcs34725::agc_lst[] = {
  { TCS34725_GAIN_60X, TCS34725_INTEGRATIONTIME_700MS,     0, 20000 },
  { TCS34725_GAIN_60X, TCS34725_INTEGRATIONTIME_154MS,  4990, 63000 },
  { TCS34725_GAIN_16X, TCS34725_INTEGRATIONTIME_154MS, 16790, 63000 },
  { TCS34725_GAIN_4X,  TCS34725_INTEGRATIONTIME_154MS, 15740, 63000 },
  { TCS34725_GAIN_1X,  TCS34725_INTEGRATIONTIME_154MS, 15740, 0 }
};
tcs34725::tcs34725() : agc_cur(0), isAvailable(0), isSaturated(0) {
}

// initialize the sensor
boolean tcs34725::begin(void) {
  tcs = Adafruit_TCS34725(agc_lst[agc_cur].at, agc_lst[agc_cur].ag);
  if ((isAvailable = tcs.begin()))
    setGainTime();
  return(isAvailable);
}

// Set the gain and integration time
void tcs34725::setGainTime(void) {
  tcs.setGain(agc_lst[agc_cur].ag);
  tcs.setIntegrationTime(agc_lst[agc_cur].at);
  atime = int(agc_lst[agc_cur].at);
  atime_ms = ((256 - atime) * 2.4);
  switch(agc_lst[agc_cur].ag) {
  case TCS34725_GAIN_1X:
    againx = 1;
    break;
  case TCS34725_GAIN_4X:
    againx = 4;
    break;
  case TCS34725_GAIN_16X:
    againx = 16;
    break;
  case TCS34725_GAIN_60X:
    againx = 60;
    break;
  }
}
  

// Retrieve data from the sensor and do the calculations
void tcs34725::getData(void) {
  // read the sensor and autorange if necessary
  tcs.getRawData(&r, &g, &b, &c);
  while(1) {
    if (agc_lst[agc_cur].maxcnt && c > agc_lst[agc_cur].maxcnt)
      agc_cur++;
    else if (agc_lst[agc_cur].mincnt && c < agc_lst[agc_cur].mincnt)
      agc_cur--;
    else break;
    setGainTime();
    delay((256 - atime) * 2.4 * 2); // shock absorber
    tcs.getRawData(&r, &g, &b, &c);
    break;
  }

  // DN40 calculations
  ir = (r + g + b > c) ? (r + g + b - c) / 2 : 0;
  r_comp = r - ir;
  g_comp = g - ir;
  b_comp = b - ir;
  c_comp = c - ir;
  cratio = float(ir) / float(c);

  saturation = ((256 - atime) > 63) ? 65535 : 1024 * (256 - atime);
  saturation75 = (atime_ms < 150) ? (saturation - saturation / 4) : saturation;
  isSaturated = (atime_ms < 150 && c > saturation75) ? 1 : 0;
  cpl = (atime_ms * againx) / (TCS34725_GA * TCS34725_DF);
  maxlux = 65535 / (cpl * 3);
  lux = (TCS34725_R_Coef * float(r_comp) + TCS34725_G_Coef * float(g_comp) + TCS34725_B_Coef * float(b_comp)) / cpl;
  ct = TCS34725_CT_Coef * float(b_comp) / float(r_comp) + TCS34725_CT_Offset;
}

tcs34725 rgb_sensor;

void setup() {
    Serial.begin(115200);
//    while(!Serial);

    pinMode(LEDpin, OUTPUT);    // LED
    pinMode(storeIT, INPUT_PULLUP);	  // Store it to file
  
    digitalWrite(LEDpin, LOW);  // turn it off

 	  // see if the card is present and can be initialized:
	  if (!SD.begin(chipSelect)){
		  Serial.println("Card failed, or not present");
		  // don't do anything more:
		  while (1);
	  }
    delay(1000);
	  MISCFile = SD.open("Color.txt", FILE_WRITE);
    if (MISCFile){
      Serial.println("MISCFile opened");
	    MISCFile.println("No:, RED, GRN, BLU, LUX, ColorTemp");
	    MISCFile.close();
    }
 
  rgb_sensor.begin();

  tft.begin();
  tft.setRotation(1);
  tft.fillScreen(HX8357_BLACK); // Clear screen set to black backgroung
  drawGraph(); // draws the graph
}
  

void loop(void) {
  rgb_sensor.getData();
  Serial.print(F("Gain:"));
  Serial.print(rgb_sensor.againx);
  Serial.print(F("x "));
  Serial.print(F("Time:"));
  Serial.print(rgb_sensor.atime_ms);
  Serial.print(F("ms (0x"));
  Serial.print(rgb_sensor.atime, HEX);
  Serial.println(F(")"));

  Serial.print(F("Raw R:"));
  Serial.print(rgb_sensor.r);
  Serial.print(F(" G:"));
  Serial.print(rgb_sensor.g);
  Serial.print(F(" B:"));
  Serial.print(rgb_sensor.b);
  Serial.print(F(" C:"));
  Serial.println(rgb_sensor.c);

  Serial.print(F("IR:"));
  Serial.print(rgb_sensor.ir);
  Serial.print(F(" CRATIO:"));
  Serial.print(rgb_sensor.cratio);
  Serial.print(F(" Sat:"));
  Serial.print(rgb_sensor.saturation);
  Serial.print(F(" Sat75:"));
  Serial.print(rgb_sensor.saturation75);
  Serial.print(F(" "));
  Serial.println(rgb_sensor.isSaturated ? "*SATURATED*" : "");

  Serial.print(F("CPL:"));
  Serial.print(rgb_sensor.cpl);
  Serial.print(F(" Max lux:"));
  lux = rgb_sensor.maxlux;
  Serial.println(lux);
  red = rgb_sensor.r_comp;
  grn = rgb_sensor.g_comp;
  blu = rgb_sensor.b_comp;

  Serial.print(F("Compensated R:"));
  Serial.print(red);
  Serial.print(F(" G:"));
  Serial.print(grn);
  Serial.print(F(" B:"));
  Serial.print(blu);
  Serial.print(F(" C:"));
  Serial.println(rgb_sensor.c_comp);

  Serial.print(F("Lux:"));
  Serial.print(rgb_sensor.lux);
  Serial.print(F(" CT:"));
  colorTemp = rgb_sensor.ct;
  Serial.print(colorTemp);
				   
  Serial.println(F("K"));

  Serial.println();
  graph(); // graphs the values on the graph
  
  if (!digitalRead(storeIT)) writeFile();
  delay(1000);
}

void drawGraph(){

  // draw title
  tft.setCursor(10, 10);
  tft.setTextColor(HX8357_RED);
  tft.setTextSize(4);
  tft.println("COLOR");

  // draw labels
  tft.setCursor(20, 300);
  tft.setTextColor(HX8357_RED);
  tft.setTextSize(3);
  tft.println("RED");
  tft.setCursor(110, 300);
  tft.setTextColor(HX8357_GREEN);
  tft.setTextSize(3);
  tft.println("GRN");
  tft.setCursor(200, 300);
  tft.setTextColor(HX8357_BLUE);
  tft.setTextSize(3);
  tft.println("BLU");
  tft.setCursor(280, 300);
  tft.setTextColor(HX8357_YELLOW);
  tft.setTextSize(3);
  tft.println("LUX");

// Color Temp Display
  int movx = 280;	// 17 steps of 11 = 187 or finish 467
  int topY = 5;
  int widx = 10;
  int heighty = 50;
  // Academo.org
  tft.fillRect(movx,topY,widx,heighty,tft.color565(255,108,0));  // Kelvin 1500
  movx = movx + widx +1;
  tft.fillRect(movx,topY,widx,heighty,tft.color565(255,137,14)); // Kelvin 2000
  movx = movx + widx +1;
  tft.fillRect(movx,topY,widx,heighty,tft.color565(255,159,70)); // Kelvin 2500
  movx = movx + widx +1;
  tft.fillRect(movx,topY,widx,heighty,tft.color565(255,177,110)); // Kelvin 3000
  movx = movx + widx +1;
  tft.fillRect(movx,topY,widx,heighty,tft.color565(255,192,141)); // Kelvin 3500
  movx = movx + widx +1;
  tft.fillRect(movx,topY,widx,heighty,tft.color565(255,206,166)); // Kelvin 4000
  movx = movx + widx +1;
  tft.fillRect(movx,topY,widx,heighty,tft.color565(255,218,187)); // Kelvin 4500
  movx = movx + widx +1;
  tft.fillRect(movx,topY,widx,heighty,tft.color565(255,228,206)); // Kelvin 5000
  movx = movx + widx +1;
  tft.fillRect(movx,topY,widx,heighty,tft.color565(255,237,222)); // Kelvin 5500
  movx = movx + widx +1;
  tft.fillRect(movx,topY,widx,heighty,tft.color565(255,246,237)); // Kelvin 6000
  movx = movx + widx +1;
  tft.fillRect(movx,topY,widx,heighty,tft.color565(255,254,250)); // Kelvin 6500
  movx = movx + widx +1;
  tft.fillRect(movx,topY,widx,heighty,tft.color565(243,242,255)); // Kelvin 7000
  movx = movx + widx +1;
  tft.fillRect(movx,topY,widx,heighty,tft.color565(230,235,255)); // Kelvin 7500
  movx = movx + widx +1;
  tft.fillRect(movx,topY,widx,heighty,tft.color565(221,230,255)); // Kelvin 8000
  movx = movx + widx +1;
  tft.fillRect(movx,topY,widx,heighty,tft.color565(215,226,255)); // Kelvin 8500
  movx = movx + widx +1;
  tft.fillRect(movx,topY,widx,heighty,tft.color565(210,223,255)); // Kelvin 9000
  movx = movx + widx +1;
  tft.fillRect(movx,topY,widx,heighty,tft.color565(205,220,255)); // Kelvin 9500
  movx = movx + widx +1;
  tft.fillRect(movx,topY,widx,heighty,tft.color565(202,218,255)); // Kelvin 10000


}

void graph(){

uint16_t y0r, y0b,y0g, y0l, hr, hb, hg, hl;
int movx;

//void fillRect(uint16_t x0, uint16_t y0, uint16_t w, uint16_t h, uint16_t color);
// Width is 80 y0 + h =  285 top y = 50 
// Algebra Val*x + b = y0
// if Val = MaxV then y0 = 50    MaxV*x + b = 50
// if Val = 0 then y0 = 235     0x + b = 235  b = 235
// solve x = -185.0/MaxV
// y0 = Val *(-185.0/MaxV) +235
// h = 285 - y0

// Fill in RGB
y0r = red*(-185.0/MaxV) +235;
hr = 285 - y0r;
y0g = grn*(-185.0/MaxV) +235;
hg = 285 - y0g;
y0b = blu*(-185.0/MaxV) +235;
hb = 285 - y0b;
y0l = lux*(-185.0/MaxV) + 235;
hl = 285 - y0l;

tft.fillRect(10,50,80,250,HX8357_BLACK);  //Clear previous
tft.fillRect(100,50,80,250,HX8357_BLACK);
tft.fillRect(190,50,80,250,HX8357_BLACK);
tft.fillRect(280,65,80,235,HX8357_BLACK);

tft.fillRect(10,y0r,80,hr,HX8357_RED);
tft.fillRect(100,y0g,80,hg,HX8357_GREEN);
tft.fillRect(190,y0b,80,hb,HX8357_BLUE);
tft.fillRect(280,y0l,80,hl,HX8357_YELLOW);

// Fill in values
  tft.setTextColor(HX8357_BLACK, HX8357_WHITE);
  tft.setTextSize(2);
  tft.setCursor(10,260);
  tft.println(red);
  tft.setCursor(100,260);
  tft.println(grn);
  tft.setCursor(190,260);
  tft.println(blu);
  tft.setCursor(280,260);
  tft.println(lux);

  // Pointer to color Temp
  // void fillTriangle(uint16_t x0, uint16_t y0, uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2, uint16_t color);
  tft.fillRect(280,60,192,22,HX8357_BLACK);  // Clear pointer space

  if (colorTemp < 1500) colorTemp = 1500;
  tft.setTextSize(3); 
  tft.setCursor(320,90);
  tft.println(colorTemp);
  tft.setCursor(320,140);
  tft.println(counter);

  movx = (int)(0.022 * colorTemp) + 252;
  tft.fillTriangle(movx, 60, movx-5, 80, movx + 5, 80, HX8357_WHITE);
}
void writeFile(){
  MISCFile = SD.open("Color.txt", FILE_WRITE);
  if (MISCFile){
    Serial.println("MISCFile opened");
  	MISCFile.print(counter++);	// file counter
  	MISCFile.print(',');
  	MISCFile.print(red);
  	MISCFile.print(',');
  	MISCFile.print(grn);
  	MISCFile.print(',');
  	MISCFile.print(blu);
  	MISCFile.print(',');
  	MISCFile.print(lux);
  	MISCFile.print(',');
  	MISCFile.println(colorTemp);
  	MISCFile.close();
  }
 Serial.println("Wrote to SD");
}
